#include "solution.hpp"
#include <gtest/gtest.h>

namespace sandbox {
    int foo() {
        return 0xF00;
    }

    bool g(void* p) {
        return p!=nullptr;
    }

    int h(int a, int b) {
        return std::max(a,b);
    }
}

#ifdef SET10_IMPLEMENTED
TEST(FunctionArguments, set10) {
    try {
        int a = 0;
        my::set10(a);
        ASSERT_EQ(a,10);
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef SET20_IMPLEMENTED
TEST(FunctionArguments, set20) {
    try {
        int a = 0;
        my::set20(&a);
        ASSERT_EQ(a,20);
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef SET50_IMPLEMENTED
TEST(FunctionArguments, set50) {
    try {
        int a = 0;
        int* pa = &a;
        my::set50(&pa);
        ASSERT_EQ(a,50);
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef SET_HELLOWORLD_IMPLEMENTED
TEST(FunctionArguments, setHelloWorld) {
    try {
        std::string s = "some string";
        my::setHelloWorld(s);
        ASSERT_STREQ(s.c_str(),"Hello, World!");
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef SET_BADBEEF_IMPLEMENTED
TEST(FunctionArguments, setBadBeef) {
    try {
        int x;
        int* px = &x;
        my::setBadBeef(&px);
        ASSERT_EQ(px,reinterpret_cast<int*>(0xBADBEEF));
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef SET_BADCODE_IMPLEMENTED
TEST(FunctionArguments, setBadCode) {
    try {
        int x;
        int* px = &x;
        my::setBadCode(px);
        ASSERT_EQ(px,reinterpret_cast<int*>(0xBADC0DE));
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef CALL_FUNCTION1_IMPLEMENTED
TEST(FunctionArguments, callFunction1) {
    try {
        ASSERT_EQ(my::callFunction1(sandbox::foo),0xF00);
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef CALL_FUNCTION2_IMPLEMENTED
TEST(FunctionArguments, callFunction2) {
    try {
        int x;
        ASSERT_TRUE(my::callFunction2(sandbox::g, &x));
        ASSERT_FALSE(my::callFunction2(sandbox::g,nullptr));
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef CALL_FUNCTION3_IMPLEMENTED
TEST(FunctionArguments, callFunction3) {
    try {
        ASSERT_EQ(my::callFunction3(sandbox::h,2,3,1,5),8);
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif

#ifdef CALL_FUNCTION4_IMPLEMENTED
TEST(FunctionArguments, callFunction4) {
    try {
        ASSERT_EQ(my::callFunction4(sandbox::foo),0xF00);
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}
#endif
